from dataclasses import dataclass, asdict, replace
from typing import List, Dict, Optional, Tuple

@dataclass
class PublicMetrics:
    reply_count: Optional[int] = None
    quote_count: Optional[int] = None
    retweet_count: Optional[int] = None
    like_count: Optional[int] = None
    
    # may be video only
    view_count: Optional[int] = None
    
    
    def items (self):
        return asdict(self).items()
    
@dataclass
class NonPublicMetrics:
    impression_count: Optional[int] = None
    user_profile_clicks: Optional[int] = None
    url_link_clicks: Optional[int] = None
    
    def items (self):
        return asdict(self).items()

@dataclass
class MediaItem:
    type: str
    preview_image_url: str
    
    media_key: Optional[str] = None
    image_url: Optional[str] = None
    
    url: Optional[str] = None
    content_type: Optional[str] = None
    duration_ms: Optional[int] = None
    
    height: Optional[int] = None
    width: Optional[int] = None
    
    size: Optional[int] = None
    
    public_metrics: Optional[PublicMetrics] = None

@dataclass
class Card:
    display_url: Optional[str] = None
    source_url: Optional[str] = None
    content: Optional[str] = None
    title: Optional[str] = None


@dataclass
class FeedItemAction:
    route: str
    route_params: Dict

@dataclass
class FeedItemAttachment:
    name: str
    content_type: str
    
    url: Optional[str] = None
    content: Optional[object] = None
    
    size: Optional[int] = None

@dataclass
class UnrepliedSection:
    feed_item_id: Optional[str] = None
    description: Optional[str] = None
    text: Optional[str] = None
    span: Optional[Tuple[int, int]] = None
    
    def text_from (self, feed_item: 'FeedItem') -> str:
        if not self.span or not feed_item.text:
            return
        
        return feed_item.text[self.span[0]:self.span[1]]

ReplyingToSection = UnrepliedSection

@dataclass
class FeedItem:
    id: str
    created_at: str
    
    display_name: str
    handle: str
    
    text: Optional[str] = None
    html: Optional[str] = None
    
    author_is_verified: Optional[bool] = None
    url: Optional[str] = None
    conversation_id: Optional[str] = None
    
    avi_icon_url: Optional[str] = None
    
    author_url: Optional[str] = None
    author_id: Optional[str] = None
    
    source_url: Optional[str] = None
    source_author_url: Optional[str] = None
    
    reply_depth: Optional[int] = 0
    
    is_marked: Optional[bool] = None
    
    card: Optional[Card] = None
    
    public_metrics: Optional[PublicMetrics] = None
    non_public_metrics: Optional[NonPublicMetrics] = None
    
    retweeted_tweet_id: Optional[str] = None
    
    source_retweeted_by_url: Optional[str] = None
    retweeted_by: Optional[str] = None
    retweeted_by_url: Optional[str] = None
    
    videos: Optional[List[MediaItem]] = None
    photos: Optional[List[MediaItem]] = None
    
    quoted_tweet_id: Optional[str] = None
    quoted_tweet: Optional['FeedItem'] = None
    
    replied_tweet_id: Optional[str] = None
    replied_tweet: Optional['FeedItem'] = None
    
    note: Optional[str] = None
    
    debug_source_data: Optional[Dict] = None
    
    attachments: Optional[List[FeedItemAttachment]] = None
    
    # At some point we may move to feed_item_actions when the set is known
    actions: Optional[Dict[str, FeedItemAction]] = None
    
    # This is a TBD concept to highlight unreplied parts of a message.
    unreplied: Optional[List[UnrepliedSection]] = None
    
    # This is a TBD concept to highlight parts of a message that are a reply.
    replying_to: Optional[List[ReplyingToSection]] = None

# tm = FeedItem(id="1", text="aa", created_at="fs", display_name="fda", handle="fdsafas")


@dataclass
class ThreadItem:
    feed_item: FeedItem
    children: Optional[List['ThreadItem']] = None
    parent: Optional['ThreadItem'] = None
    parents: Optional[List['ThreadItem']] = None
    
    actions: Optional[Dict[str, FeedItemAction]] = None
    


@dataclass
class FeedServiceUser:
    id: str
    url: str
    name: str # display_name
    username: str # handle
    
    created_at: Optional[str] = None
    description: Optional[str] = None
    preview_image_url: Optional[str] = None
    website: Optional[str] = None
    
    actions: Optional[Dict[str, FeedItemAction]] = None 
    
    source_url: Optional[str] = None
    
    @property
    def display_name (self) -> str:
        return self.name
        
    @property
    def handle (self) -> str:
        return self.username

def cleandict(d):
    if isinstance(d, dict):
        return {k: cleandict(v) for k, v in d.items() if v is not None}
    elif isinstance(d, list):
        return [cleandict(v) for v in d]
    else:
        return d
